from cocos.layer import Layer
from cocos.scene import Scene
from cocos.tiles import MapLayer
from cocos.tiles import RectCell
from cocos.actions import *

import logging
log = logging.getLogger('Board')

#
# Local imports
#
from cell import *
from shape import *


class Board(Layer):
    is_event_handler = False
    def __init__(self, w, h, premade = None):
        super(Board,self).__init__()
        
        self.cells = {}
        
        self.boardWidth = w
        self.boardHeight = h
        
        self.SetupBoard()
        if (premade != None):
            self.SetupPremade(premade)
        
    def SetupBoard(self):
        self.cells = {}
        for x in range( self.boardWidth ):
            for y in range( self.boardHeight ):
                self.cells[ (x,y) ] = None
        

        log.info("Cells: "+str(self.boardWidth)+":"+str(self.boardHeight))

                
    def SetupPremade(self, premade):
        
        for x in range( self.boardWidth ):
            for y in range( self.boardHeight ):
                if (premade[x][y] != 0):
                    cell =  MakeCell(premade[x][y]) 
                    cell.ConvertCellPosToPosition(x, y)
                    self.add(cell)
                    self.cells[ (x,y) ] = cell
                else:
                    self.cells[ (x,y) ] = None
                    
    def SetupPremadeLevel(self, premade):
        celllist = []
        if premade == None:
            return []
        
        for x in range( self.boardWidth ):
            for y in range( self.boardHeight ):
                if (premade[x][y] != 0):
                    cell =  MakeCell(premade[x][y]) 
                    cell.ConvertCellPosToPosition(x, y)
                    offset_x = 32+16+2
                    offset_y = 136#-32+2
                    cell.x = int(offset_x+x*32)
                    cell.y = 600-int(offset_y+y*32)
                    self.cells[ (x,y) ] = cell
                    celllist.append(cell)


        return celllist
  
    def ShifCellDown(self, x, y):
        new = (x,y)
        old = (x,y-1)
        self.cells[ new ] = self.cells[ old ]
        self.cells[ old ] = None
        
        if (  self.cells[ new ] != None  ):

            offset_x = 32+16+2
            offset_y = 136#-32+2
     
            self.cells[ new ].x = int(offset_x+x*32)
            self.cells[ new ].y = 600-int(offset_y+y*32)
        
    def ApplyGravity(self, x):

        y = self.boardHeight-1
        while y > 0 :
            newpos = (x,y)
            oldpos = (x,y-1)
            #log.info("Checking gravity for "+str(oldpos) +"-->"+str(newpos))
            if (self.cells[ newpos ] == None):
                
                self.ShifCellDown(x, y) 
                    
                
                yy = y-1
                while yy > 0 :
                    self.ShifCellDown(x, yy)
                    yy -= 1
                    
                if (self.cells[ newpos ] == None): 
                    self.ShifCellDown(x, y) 
             

            y -= 1
    
    def GetColumn(self, col):
        if (col <0 or col >= self.boardWidth):
            return None
        
        return self.cells[col]    
    
    def GetCell(self, pos):
        if (pos[0] <0 or pos[0] >= self.boardWidth or pos[1] <0 or pos[1]>=self.boardHeight):
            return None
        
        return self.cells[pos]
    
    def GetCellID(self, pos):
        
        cell = self.GetCell(pos)
        if (cell == None):
            return 0
        
        return cell.cell_ID
    
    def SetCell(self, pos, value):
        #log.info("SetCell: "+str(pos)+" "+str(value))
        self.cells[pos] = value
        
    def Clear(self):
        for x in range( self.boardWidth ):
            for y in range( self.boardHeight ):
                 self.cells[ (x,y) ] = None
                 
    def Match(self):
        for x in range( self.boardWidth ):
            for y in range( self.boardHeight ):
                if (self.cells[ (x,y) ] != None):
                    self.cells[ (x,y) ].do(ScaleBy(5,1.5 )) 
                