import math
import random
import string
import ConfigParser


from pyglet import clock

from pyglet.window import key

# Cocos2d
from cocos.director import director
from cocos.tiles import ScrollableLayer
from cocos.layer import Layer
from cocos.scene import Scene
#from cocos.tiles import MapLayer
#from cocos.tiles import RectCell
from cocos.actions import *
import logging
log = logging.getLogger('GameLayer')



#
# Local imports
#
from cell import *
from shape import *
from hud import HUD

from gameoverscreen import GameOver
from audio import AudioPlayer

import gamekeys

class GameLayer(Layer):
    is_event_handler = True
    def __init__(self, hud, level):
        super(GameLayer,self).__init__()
        self.cells = None
        self.nextCell = None
        self.paused = True
        self.gameWidth = 10
        self.gameHeight = 10
        self.combo = 0
        self.cellSize = 32
        self.defaultActiveCellPos = [4,0];
        self.activeCellPos = [0,0]
        self.lastCellPos = [0,0]
        self.activeCellPos[0] = self.defaultActiveCellPos[0];
        self.activeCellPos[1] = self.defaultActiveCellPos[1];
        self.activeCell = None#self.GetNextCell();
        self.kleft = False
        self.kright = False
        self.kdown = False
        
        self.speed = 1.0/3.0
        
        bg = Sprite("gamescreen.png")
        bg.x = 400
        bg.y = 300
        self.add(bg, z = -1)
        self.offset_x = 32+16+2
        self.offset_y = 136#-32+2
        
        self.level = None
        self.shoppingList = []
        self.cellChoices = []
        self.HUD = hud
        log.info("score set to "+str(self.HUD.score))
        self.limboCells = []
        
        path = os.path.abspath(os.path.dirname(__file__)+"/../data/levels")
        self.levelConfig = ConfigParser.SafeConfigParser()   
        self.levelConfig.read(path+"/"+str(level)+".ini")
        
        
        
        
        self.SetupBoard()
        
        
        log.info("__init__ complete")
    
    
    def SetupBoard(self):
        
        levelName = self.levelConfig.get("level","name")
        
        self.HUD.SetLevelInfo(levelName)
        self.HUD.SetSpecialsAvail(eval(self.levelConfig.get("level", "Specials")))
        
        log.info("Level name: "+levelName)
        
        self.gameWidth = self.levelConfig.getint("level", "width")
        self.gameHeight = self.levelConfig.getint("level", "height")
        
        self.speed = 1.0 / self.levelConfig.getint("level", "speed")
        
        self.cellChoices = eval(self.levelConfig.get("level", "CellTypes"))
       
        self.nextCell = self.GetNextCell();
        self.add(self.nextCell)
        
        self.cells = Board(self.gameWidth,self.gameHeight)
        cells = self.cells.SetupPremadeLevel(eval(self.levelConfig.get("level", "CELLS")))
        for cell in cells:
            self.add(cell)
            
        self.GetNewCell()
        
        self.level = [] 
        for (shapeDef, data) in self.levelConfig.items("shapes"):
            needed, special = eval(data)
            shapeObj = MakeShape(string.capwords(shapeDef))
            #print string.capwords(shapeDef)
            shapeObj.SetNeeded(needed, special)
            self.level.append(shapeObj)
            
        x = 569
        y = 440
        log.info("level:"+str(self.level))

        for shape in self.level:
            shape.x = x
            shape.y = y
            y -= (shape.boardHeight+1)*32 
            self.add(shape)       
        
        log.info("SetupBoard complete")
   
    def GetNewCell(self):
        log.info("GetNewCell "+str(self.activeCellPos) + " :" +  repr(self.cells.GetColumn([self.activeCellPos[0]])))
        
        self.activeCellPos[0] = self.defaultActiveCellPos[0];
        self.activeCellPos[1] = self.defaultActiveCellPos[1];
        self.lastCellPos = [self.activeCellPos[0], self.activeCellPos[1]]
        
        if ((self.cells.GetCell((self.activeCellPos[0], self.activeCellPos[1])) != None) and (self.cells.GetCell((self.activeCellPos[0],self.activeCellPos[1]+1)) != None)):
            log.info("GetNewCell GAME OVER")
            self.unschedule(self.UpdateCells)
            self.parent.add(GameOver(False, self.HUD.score), z=100)
            return
        
        self.activeCell = self.nextCell
        
        #self.SetCellPos(int(self.activeCellPos[0]), int(self.activeCellPos[1]))
            
        self.nextCell = self.GetNextCell()
        self.SetNextCellPos()
        
        self.cells.add(self.nextCell)
        self.add(self.nextCell)
         
        #log.info("GetNewCell complete")
    def SetNextCellPos(self):
        self.nextCell.x = 224+16
        self.nextCell.y = 498

    def GetNextCell(self):
        log.info("GetNextCell")

        return MakeCell(random.choice(self.cellChoices))
    
    def SetCellPos(self, x, y):
        self.cells.SetCell((self.lastCellPos[0], self.lastCellPos[1]) , None)   
         
        self.activeCell.x = int(self.offset_x+x*self.cellSize)
        self.activeCell.y = 600-int(self.offset_y+y*self.cellSize)
        #log.info("SetCellPos "+str(type(dx))+","+str(type(dy))+" - " +str(type(self.activeCell.cell_ID)))
        #log.info("cells "+repr(self.cells))
        self.cells.SetCell((x,y), self.activeCell)  
        
        self.lastCellPos = (self.activeCellPos[0], self.activeCellPos[1])
        
        #log.info("SetCellPos complete"+str(dx)+","+str(dy))
       
    def UpdateCells(self,dt):

        if (self.combo >0):
            self.CheckCells()
            
        if gamekeys.keys[key.LEFT] and (not self.kleft):
            self.OnKeyLeft()
        if gamekeys.keys[key.RIGHT] and (not self.kright):
            self.OnKeyRight()
        if gamekeys.keys[key.DOWN] and (not self.kdown):
            self.OnKeyDown()

        self.kleft = False
        self.kright = False
        self.kdown = False

        if ((self.activeCellPos[1] < self.gameHeight-1) and (self.cells.GetCell((self.activeCellPos[0],self.activeCellPos[1]+1))== None)):

            self.cells.SetCell((self.activeCellPos[0],self.activeCellPos[1]), None)  
             
            self.activeCellPos[1] += 1
            #log.info("UpdateCells activeCellPos "+repr(self.activeCellPos))
        else:
            #self.SetCellPos(activeCellPos[0], activeCellPos[1])
            
            # Lets do something special
            cellid = self.cells.GetCellID((self.activeCellPos[0],self.activeCellPos[1]))
            if (cellid >= SPECIALS_OFFSET):
                #self.cells.SetCell((self.activeCellPos[0],self.activeCellPos[1]), None)
                # do stuff
                self.ActivateSpecial(cellid-SPECIALS_OFFSET)
            else:
                AudioPlayer[0].PlaySound("thunk.ogg")

                  
            self.CheckCells()
            self.GetNewCell()
            
        self.SetCellPos(self.activeCellPos[0], self.activeCellPos[1])

        #log.info("UpdateCells complete")
     
    def CheckCells(self):
        found = None
        pos = None
        log.info("CheckCells: "+str(self.cells))
        for target in self.level:
            pos = target.CompareShape(self.cells)
            if (pos != None):
                found = target
                break
        
        if (found != None):
            
            self.combo += 1
            log.info("Waaah found "+found.name+" "+str(self.combo))
            
            if (found.UseShape()):
                self.remove(found)               
                self.level.remove(found)
                
            self.HUD.Matched(found, self.combo)
            # Do we clear now we have matched?
            self.ClearShape( found, pos )
            
            shape = MakeShape(found.name)
            shape.x = self.offset_x +pos[0]*32
            shape.y = 600 - (self.offset_y +pos[1]*32)
            shape.numberNeededText.element.text = ""
            shape.Match()
            
            shape.do(FadeOut(0.5) + CallFunc( self.ClearLimbo, shape ) )
            lab = Label(found.name+" "+str(found.points), font_size=10,font_name= "M01_CPS")
            lab.x = -40
            lab.y = 50
            shape.add(lab, z=10)
            
            self.add(shape)
            
            self.CheckForVictory()
        else:
            self.combo = 0
      
    def ActivateSpecial(self, specialID):
                
        if (specialID == 0):
            choices = eval(self.levelConfig.get("level", "Specials"))
            choices.remove(0)
            
            specialID = random.choice(choices)
            log.info("ActivateSpecial randomising...")
            
        log.info("ActivateSpecial "+str(specialID))
        
         # Remove the speicial
        self.ClearCell((self.activeCellPos[0],self.activeCellPos[1]), True)
       
        # The following is instead of a switch
        {
                  1:self._BombSpecial,
                  2:self._BulldozerSpecial,
                  3:self._FloodSpecial,
                  4:self._TNTSpecial,
                  5:self._TornadoSpecial,  
        }[specialID]()
        
        

    def _BombSpecial(self):
        AudioPlayer[0].PlaySound("nuke.ogg")
        self.parent.do( ShakyTiles3D( randrange=6, grid=(16,16), duration=1)  + StopGrid() )
        log.info("Bomb")
        # Clear everything
        x = 0
        while x < self.cells.boardWidth:
            y = 0
            while y < self.cells.boardHeight:
                self.ClearCell((x,y), True)
                y += 1
            x += 1
            
    def _BulldozerSpecial(self):
        AudioPlayer[0].PlaySound("bulldozer.ogg")
        self.parent.do( Shaky3D( randrange=1, grid=(16,8), duration=1)  + StopGrid() )
        
        log.info("Bulldozer @"+str((self.activeCellPos[0],self.activeCellPos[1])))

        self.ClearCell((self.activeCellPos[0] + 1,self.activeCellPos[1]), True)
        self.ClearCell((self.activeCellPos[0] -1,self.activeCellPos[1]), True)
        self.cells.ApplyGravity(self.activeCellPos[0] + 1)
        self.cells.ApplyGravity(self.activeCellPos[0])
        self.cells.ApplyGravity(self.activeCellPos[0] -1)
        
    def _FloodSpecial(self):
        AudioPlayer[0].PlaySound("flood.ogg")
        log.info("Flood @"+str(self.activeCellPos[1]))
        
        #wave =  Waves( waves=2, hsin=True, vsin=True, grid=(16,10), duration=0.5)
        self.parent.do( Liquid( waves=3, grid=(16,16), duration=1)  + StopGrid() )

        x = 0
        while x < self.cells.boardWidth:
            self.ClearCell((x,self.activeCellPos[1]), True)
            self.cells.ApplyGravity(x)#, self.activeCellPos[1])
            x += 1
            
    def _TNTSpecial(self):
        AudioPlayer[0].PlaySound("dynamite.ogg")
        self.parent.do( Shaky3D( randrange=2, grid=(16,16), duration=0.2)  + StopGrid() )
        log.info("TNT @"+str((self.activeCellPos[0],self.activeCellPos[1])))
        
        self.ClearCell((self.activeCellPos[0] + 1,self.activeCellPos[1]), True)
        self.ClearCell((self.activeCellPos[0] - 1,self.activeCellPos[1]), True)
        self.ClearCell((self.activeCellPos[0], self.activeCellPos[1]), True)
        
        self.ClearCell((self.activeCellPos[0] + 1,self.activeCellPos[1]+1), True)
        self.ClearCell((self.activeCellPos[0] - 1,self.activeCellPos[1]+1), True)
        self.ClearCell((self.activeCellPos[0], self.activeCellPos[1]+1), True)
        
        self.ClearCell((self.activeCellPos[0] + 1,self.activeCellPos[1]-1) , True)
        self.ClearCell((self.activeCellPos[0] - 1,self.activeCellPos[1]-1) , True)
        self.ClearCell((self.activeCellPos[0], self.activeCellPos[1]-1) , True)

        
        self.cells.ApplyGravity(self.activeCellPos[0] + 1)#, self.activeCellPos[1])
        self.cells.ApplyGravity(self.activeCellPos[0])# , self.activeCellPos[1])
        self.cells.ApplyGravity(self.activeCellPos[0] - 1)#, self.activeCellPos[1])

    def _TornadoSpecial(self):
        AudioPlayer[0].PlaySound("tornado.ogg")
        self.parent.do( Twirl( twirls=0.2, center=(400,300), amplitude=6, grid=(32,16), duration=1)   + StopGrid() )
       
        log.info("Tornado @"+str(self.activeCellPos[0]))
        y = 0
        while y < self.cells.boardHeight:
            self.ClearCell((self.activeCellPos[0],y))
            y += 1

    def ClearLimbo(self, item):
        self.remove(item)   
        
    def CreateSpecial(self, activateIndex):    
        log.info("Player tried to acivate special slot #"+str(activateIndex))   
             
        active, cellID = self.HUD.IsSpecialActive(activateIndex)
        if (True):#active):
            self.HUD.SetSpecialActive(activateIndex, False)
            self.remove(self.nextCell)
            self.nextCell = MakeCell(cellID)
            self.SetNextCellPos()

            self.add(self.nextCell)
            
            
    def ClearShape(self, shape, pos):  
        '''Remove other cells from this shape'''
        sx,sy = pos
        #log.info("ClearShape: "+str(shape)+" at "+str((sx,sy))+" shape: "+str((shape.boardWidth,shape.boardHeight)))
        
        while sx-pos[0] < shape.boardWidth:
            sy = pos[1]
            while sy-pos[1] < shape.boardHeight:
                self.ClearCell((sx,sy))
                sy += 1
            self.cells.ApplyGravity(sx)#, sy)
            sx += 1
            
    def ClearCell(self, pos, shrink = False):   
        if (pos[0] <0 or pos[1] <0 or pos[0] >= self.cells.boardWidth or pos[1] >= self.cells.boardHeight):
            log.error("ClearCell WTF?")
            return
             
        cell = self.cells.GetCell(pos)
        if (cell != None):
            
            self.limboCells.append(cell)
            if (shrink):
                cell.do(ScaleTo( 0.01, 0.8 ) + CallFunc( self.ClearLimbo, cell ) )
            else:
                #cell.add(Label("moo"))
                #cell.do(ScaleTo( 1.3, 0.8 ) + CallFunc( self.ClearLimbo, cell ) )
                cell.do(FadeOut( 0.8 ) + CallFunc( self.ClearLimbo, cell ) )
            # Clear cell
            self.cells.SetCell(pos, None)

    def CheckForVictory(self):
        # what are the victory conditions?
        # 1. finish shopping list
        # 2. board clear
        needed = 0
        for shape in self.level:
            needed += shape.numberNeeded

        if (needed == 0):
            # Win on isle 1!
            self.unschedule(self.UpdateCells)
            self.parent.add(GameOver(True, self.HUD.score), z=100)

    def StartGame(self):
        self.schedule_interval(self.UpdateCells,self.speed)
        self.paused = False
        
    def PauseGame(self):
        self.unschedule(self.UpdateCells)
        self.paused =  True

    def OnKeyRight(self):
        if ((self.activeCellPos[0]+1 <self.gameWidth) and self.cells.GetCell((self.activeCellPos[0]+1,self.activeCellPos[1])) == None):
            self.activeCellPos[0] +=1
            self.SetCellPos(self.activeCellPos[0], self.activeCellPos[1])
            
            
    def OnKeyLeft(self):        
        if ((self.activeCellPos[0]-1 >=0) and self.cells.GetCell((self.activeCellPos[0]-1,self.activeCellPos[1])) == None):
            self.activeCellPos[0] -=1
            self.SetCellPos(self.activeCellPos[0], self.activeCellPos[1])
            
            
    def OnKeyDown(self):        
        if ((self.activeCellPos[1] < self.cells.boardHeight -1) and self.cells.GetCell((self.activeCellPos[0],self.activeCellPos[1]+1)) == None):
            self.activeCellPos[1] +=1
            self.SetCellPos(self.activeCellPos[0], self.activeCellPos[1])
            
            
    def on_key_press (self, keys, modifiers):
         
        if (keys == key.P ):
            if (self.paused):
                self.StartGame()
            else:
                self.PauseGame()
            return
               
        if (self.paused):
            return
        
        # Prehaps we can rely on held keys only? hmm not sure prob not   
        if (keys == key.RIGHT ):
            self.OnKeyRight()
            self.kright = True
            return
        
        if (keys == key.LEFT ):
            self.OnKeyLeft()
            self.kleft = True
            return
        
        if (gamekeys.keys == key.DOWN ):
            self.OnKeyDown()
            self.kdown = True
            return
        
        ''' 
        if (keys == key.R ):
            log.info("Restart requested.")
            #children = self.get_children
            self.SetupBoard()
            self.StartGame()
            return
        
        if (keys == key.U ):
            self.UpdateCells()
            return

          
            
        if (keys == key.F1):
            log.info("Winning game. score: "+str(self.HUD.score))
            self.unschedule(self.UpdateCells)
            self.parent.add(GameOver(True, self.HUD.score), z=100)
            
        if (keys == key.F2):
            log.info("Lose game. score: "+str(self.HUD.score))
            self.unschedule(self.UpdateCells)
            self.parent.add(GameOver(False, self.HUD.score), z=100)
            
        if (keys == key.F3):
            self.HUD.SetSpecialActive(2, True)
        '''
            
        if (keys == key._1 ):
            self.CreateSpecial(0)
            return
        if (keys == key._2 ):
            self.CreateSpecial(1)
            return
        if (keys == key._3 ):
            self.CreateSpecial(2)
            return
        if (keys == key._4 ):
            self.CreateSpecial(3)
            return
        if (keys == key._5 ):
            self.CreateSpecial(4)
            return
        if (keys == key._6 ):
            self.CreateSpecial(5)
            return
            

    def on_key_release (self, key, modifiers):
        pass
        #for player in gPlayers:
        #    player.KeyRelease(key)

    def on_enter(self):
        super( GameLayer, self).on_enter()
        self.is_event_handler = True
        self.StartGame()
        
    def on_exit(self):
        super( GameLayer, self).on_exit()
        self.is_event_handler = False        
        self.unschedule(self.UpdateCells)
    
