#
# Python includes
#
import os, sys
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

from cocos.director import director
from cocos.layer import Layer,ColorLayer
from cocos.scene import Scene
from cocos.sprite import Sprite
from cocos.scenes.transitions import *
from cocos.actions import *

from cocos.menu import *
from cocos.text import *

from levelcomplete import LevelSelectScene
from imagelayer import ImageLayer
from backmenu import BackMenu
from audio import AudioPlayer
               
class MainMenu( Menu ):

    def __init__(self):
        super( MainMenu, self).__init__() 
        
        self.font_title['color'] = (0,170,0,255)
        self.font_title['font_size'] = 20
        self.font_title['font_name'] = "M01_CPS"
        self.font_title['x'] = 400
        self.font_title['y'] = 200
        
        self.font_item['color'] = (255,255,255,255)
        self.font_item['font_size'] = 10
        self.font_item['font_name'] = "M01_CPS"
        
        self.font_item_selected['color'] = (0,200,0,255)
        self.font_item_selected['font_size'] = 12
        self.font_item_selected['font_name'] = "M01_CPS"

        self.menu_valign = BOTTOM
        self.menu_halign = CENTER

        items = []

        items.append( MenuItem('Play', self.on_play) )
        items.append( MenuItem('Instructions', self.on_instructions) )
        items.append( MenuItem('Credits', self.on_credits) )
        items.append( MenuItem('Quit', self.on_quit) )

        self.create_menu( items)
        
        # Space em out
        pos = 0
        for menuItem in items:
            menuItem.y += 310-pos*10
            pos += 1
            
    def on_enter(self):
        super(MainMenu, self).on_enter() 
 
               
    def on_play( self ):
        director.push(LevelSelectScene(0))

 
    def on_instructions( self ):
        instLayer = Layer()
        x,y = director.get_window_size()
        
        textMove = Label("Move", (x*0.1, y*0.5), font_size=15,font_name= "M01_CPS", color=(0,170,0,255))
        textMove2 = Label("Keys: LEFT / RIGHT / DOWN", (x*0.15, y*0.45), font_size=10, font_name= "M01_CPS")
        instLayer.add(textMove)
        instLayer.add(textMove2)

        textLaunch = Label("Launch Special", (x*0.1, y*0.4), font_size=15,font_name= "M01_CPS", color=(0,170,0,255))
        textLaunch2 = Label("Keys: 1-6", (x*0.15, y*0.35), font_size=10, font_name= "M01_CPS")
        instLayer.add(textLaunch)
        instLayer.add(textLaunch2)
        
        textRules = Label("Overview", (x*0.1, y*0.75), font_size=15,font_name= "M01_CPS", color=(0,170,0,255))
        textRules2 = Label("Combine objects to build items on the list", (x*0.15, y*0.7), font_size=10, font_name= "M01_CPS")
        textRules3 = Label("Complete enough items and you receive a special", (x*0.15, y*0.65), font_size=10, font_name= "M01_CPS")
        instLayer.add(textRules)
        instLayer.add(textRules2)
        instLayer.add(textRules3)
            
        title = Label("Instructions", font_size=20,font_name= "M01_CPS", color=(0,170,0,255))
        title.y = y*0.85
        title.x = 200
        main_scene = Scene (ImageLayer('suburbialogo.png'), BackMenu(), instLayer, title) 
        director.push(main_scene)
        
    def on_credits( self ):
        AudioPlayer[0].PlaySound("credits.ogg")
        credLayer = Layer()
        x,y = director.get_window_size()
        
        textMove = Label("smatts", (x*0.1, y*0.7), font_size=15,font_name= "M01_CPS", color=(0,170,0,255))
        textMove2 = Label("game design, art", (x*0.15, y*0.65), font_size=10, font_name= "M01_CPS")
        credLayer.add(textMove)
        credLayer.add(textMove2)

        textLaunch = Label("alia", (x*0.1, y*0.6), font_size=15,font_name= "M01_CPS", color=(0,170,0,255))
        textLaunch2 = Label("game design, programming", (x*0.15, y*0.55), font_size=10, font_name= "M01_CPS")
        credLayer.add(textLaunch)
        credLayer.add(textLaunch2)
        
        textLaunch = Label("thanks", (x*0.1, y*0.4), font_size=15,font_name= "M01_CPS", color=(0,170,0,255))
        textLaunch2 = Label("Kate & Mayumi for letting us play", (x*0.15, y*0.35), font_size=10, font_name= "M01_CPS")
        credLayer.add(textLaunch)
        credLayer.add(textLaunch2)

        title = Label("Credits", font_size=20,font_name= "M01_CPS", color=(0,170,0,255))
        title.y = y*0.85
        title.x = 200
        main_scene = Scene (ImageLayer('creditscreen.png'), BackMenu(), credLayer, title) 
        director.push(main_scene)

    def on_quit(self):
        pyglet.app.exit()

if __name__ == "__main__":


    director.init( resizable=True, width=800, height=600 )
    scene = Scene(MainMenu())
    director.run( scene )
