#
# Python includes
#
import os, sys
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

import logging
log = logging.getLogger('GameLayer')

from cocos.text import Label
from cocos.actions import *

from cell import *

from board import Board

class Shape(Board): # maybe turn this into a layer
    ''' Defines a set of cells that represent a game object such as a house '''
    def __init__(self, prefab):
        w = prefab.__len__()
        h = prefab[0].__len__()
        super(Shape,self).__init__( w, h, prefab)
        self.points = 0
        self.name = ''
        self.limitedSupply = False
        self.numberNeeded = 1
        self.specialActivated = None
        
        self.numberNeededText = Label("x", font_size=10,font_name= "M01_CPS", color=(255,255,255,255))
        self.numberNeededText.x = -60
        self.numberNeededText.y = -(h/2)*32
        
        self.add(self.numberNeededText)
        
    def SetNeeded(self, needed, special):
        self.numberNeeded = int(needed)
        self.numberNeededText.element.text = "x" + str(self.numberNeeded)
        
        seq = ScaleBy(1.1, 0.1)
        self.numberNeededText.do(seq+Reverse(seq))
        
        self.specialActivated = special
    
    def UseShape(self):
        if (self.numberNeeded > 0):
            self.SetNeeded(self.numberNeeded -1, self.specialActivated)
        
        seq = ScaleBy(1.05, 0.5)
        se2 = DoAction(seq + Reverse(seq))
        self.do(se2*2)#seq + Reverse(seq))
        
        if (self.limitedSupply and self.numberNeeded == 0):
            return True
        
        return False
        
    def CompareShape(self, arena):
        log.info("CompareShape["+self.name+"]: " + str(self.cells))
        
        width = arena.boardWidth
        maxHeight = arena.boardHeight
        myWidth = self.boardWidth
        myMaxHeight = self.boardHeight
       
        ax = 0
        ay = 0
        
        
        while ax < width - (myWidth - 1):
            ay = 0
            while ay < maxHeight - (myMaxHeight - 1):
                arenaID = arena.GetCellID((ax, ay))
                # Check the first cell
                if (((self.cells[(0,0)] == None and arenaID == 0) or (self.cells[(0,0)] != None and self.cells[(0,0)].cell_ID == arenaID))):
                    
                    # might have one....
                    x = 0
                    found = True
                    while x < myWidth:
                        y = 0
                        while y < myMaxHeight:
                            arenaID = arena.GetCellID((ax+x, ay+y))
                            if ((self.cells[(x,y)] == None and arenaID == 0) or  (self.cells[(x,y)] != None and self.cells[(x,y)].cell_ID != arenaID)):
                                found = False
                                break
                            y += 1
                        x += 1
                        
                    if (found == True):    
                        # if it gets to here then we found one
                        return [ax,ay]
                        
                ay += 1
            ax += 1
        
        # if it gets to here we didnt find one
        return None
 


def MakeShape(name):
    cellstring = elemConfig.get(name,"CELLS")
    log.info("elemConfig Got: "+cellstring)
    
    cells = eval(cellstring)
    shape = Shape(cells) 
    shape.name = name
    shape.points = elemConfig.getint(name, "points")
    return shape
 